/*******************************************************************\

Copyright (c) 2008 Leiden University (LERC group at LIACS).
All rights reserved.

The use and distribution terms for this software are covered by the 
Common Public License 1.0 (http://opensource.org/licenses/cpl1.0.txt)
which can be found in the file LICENSE at the root of this distribution.
By using this software in any fashion, you are agreeing to be bound by 
the terms of this license.

You must not remove this notice, or any other, from this software.

\*******************************************************************/

#ifndef P_3_H
#define P_3_H

#include "math.h"
#include "yapi.h"

#include "aux_func.h"
#include <iostream>


class P_3 : public Process {
  private:

    // Input Gates and controllers
    InPort<tCH_7> IG_1;
    InPort<tCH_8> IG_2;
    InPort<tCH_9> IG_3;
    InPort<tCH_10> IG_4;
    InPort<tCH_11> IG_5;
    InPort<tCH_12> IG_6;

    // Output Gates
    OutPort<tCH_14> OG_1;

    // Parameters
    int N;
    int M;

    // Input Arguments 
    tCH_7 in_0ND_2;
    tCH_8 in_1ND_2;
    tCH_9 in_2ND_2;
    tCH_10 in_3ND_2;
    tCH_11 in_4ND_2;
    tCH_12 in_5ND_2;

    // Output Arguments 
    tCH_14 out_6ND_2;

  public:

    // Constructor 
    P_3(Id n,
    In<tCH_7>& IG_1_instance,
    In<tCH_8>& IG_2_instance,
    In<tCH_9>& IG_3_instance,
    In<tCH_10>& IG_4_instance,
    In<tCH_11>& IG_5_instance,
    In<tCH_12>& IG_6_instance,
    Out<tCH_14>& OG_1_instance,
    int parm_N,
    int parm_M) :
    Process(n),
    IG_1(id("IG_1"), IG_1_instance ),
    IG_2(id("IG_2"), IG_2_instance ),
    IG_3(id("IG_3"), IG_3_instance ),
    IG_4(id("IG_4"), IG_4_instance ),
    IG_5(id("IG_5"), IG_5_instance ),
    IG_6(id("IG_6"), IG_6_instance ),
    OG_1(id("OG_1"), OG_1_instance ),
    N(parm_N),
    M(parm_M)
    {
    }; 

    const char* type() const {return "P_3";};

    void main() {

      for( int c0 =  ceil1(3); c0 <=  floor1(M ); c0 += 1 ) {
        for( int c1 =  ceil1(3); c1 <=  floor1(N ); c1 += 1 ) {
            read(IG_1, in_0ND_2);

            read(IG_2, in_1ND_2);

            read(IG_3, in_2ND_2);

            read(IG_4, in_3ND_2);

            read(IG_5, in_4ND_2);

            read(IG_6, in_5ND_2);


            _gradient(in_0ND_2, in_1ND_2, in_2ND_2, in_3ND_2, in_4ND_2, in_5ND_2, out_6ND_2) ;
            execute("_gradient");


            write( OG_1, out_6ND_2);

        } // for c1
      } // for c0

    } // main
};

#endif
