/*******************************************************************\

Copyright (c) 2008 Leiden University (LERC group at LIACS).
All rights reserved.

The use and distribution terms for this software are covered by the 
Common Public License 1.0 (http://opensource.org/licenses/cpl1.0.txt)
which can be found in the file LICENSE at the root of this distribution.
By using this software in any fashion, you are agreeing to be bound by 
the terms of this license.

You must not remove this notice, or any other, from this software.

\*******************************************************************/

#ifndef P_4_H
#define P_4_H

#include "math.h"
#include "yapi.h"

#include "aux_func.h"
#include <iostream>


class P_4 : public Process {
  private:

    // Input Gates and controllers
    InPort<tCH_13> IG_1;
    InPort<tCH_14> IG_2;

    // Output Gates
    OutPort<tCH_15> OG_1;

    // Parameters
    int N;
    int M;

    // Input Arguments 
    tCH_13 in_0ND_3;
    tCH_14 in_1ND_3;

    // Output Arguments 
    tCH_15 out_2ND_3;

  public:

    // Constructor 
    P_4(Id n,
    In<tCH_13>& IG_1_instance,
    In<tCH_14>& IG_2_instance,
    Out<tCH_15>& OG_1_instance,
    int parm_N,
    int parm_M) :
    Process(n),
    IG_1(id("IG_1"), IG_1_instance ),
    IG_2(id("IG_2"), IG_2_instance ),
    OG_1(id("OG_1"), OG_1_instance ),
    N(parm_N),
    M(parm_M)
    {
    }; 

    const char* type() const {return "P_4";};

    void main() {

      for( int c0 =  ceil1(3); c0 <=  floor1(M ); c0 += 1 ) {
        for( int c1 =  ceil1(3); c1 <=  floor1(N ); c1 += 1 ) {
            read(IG_1, in_0ND_3);

            read(IG_2, in_1ND_3);


            _absVal(in_0ND_3, in_1ND_3, out_2ND_3) ;
            execute("_absVal");


            write( OG_1, out_2ND_3);

        } // for c1
      } // for c0

    } // main
};

#endif
